==> trip-0.1.0.tar.gz: trip-0.1.0/tripstack/tripcore/TRIPCommunity.cxx <==
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TRIPCommunity_cxx_Version = 
"$Id: TRIPCommunity.cxx,v 1.2 2000/11/13 22:03:32 kevin Exp $";

#ifdef __linux__
    #include <algo.h>
#else
    #include <algorithm>
#endif

#include "TRIPCommunity.hxx"
#include "TRIPUtil.hxx"
#include "cpLog.h"


TRIPCommunity::TRIPCommunity(const TRIPCommunity& other) 
{
     copyObj(other);
}

TRIPCommunity& 
TRIPCommunity::operator=(const TRIPCommunity& other)
{
    if(this != &other)
    {
        copyObj(other);
    }
    return(*this);
}

void 
TRIPCommunity::dump(ostream& os) const
{
    os.width(25);
    os << "**Community**" << endl;
    os.width(25);
    int idx = 0;
    for(CommunityList::const_iterator itr = _commList.begin();
               itr != _commList.end(); itr++)
    {
        os << hex << static_cast<int>(*itr) << ",";
        idx++;
        if(idx > 5)
        {
            //Print no more than 5 communities in a line
            os << endl;
            os.width(25);
            idx = 0;
        }
    }
}

void 
TRIPCommunity::writeData(int id) const throw(VNetworkException&)
{
    TRIPRouteAttr::writeData(id);
    assert(_commList.size());
    u_int32_t* cArray  = new u_int32_t[_commList.size()];
    int i = 0;
    for(CommunityList::const_iterator itr = _commList.begin();
               itr != _commList.end(); itr++)
    {
        cArray[i++] = (*itr);
    }
    TRIPUtil::writeData(id, cArray, sizeof(u_int32_t)*_commList.size());

    delete []cArray;
}

TRIPCommunity::TRIPCommunity(const char* buf) 
        throw (TRIPBadDataException&)
    : TRIPRouteAttr(buf)
{
    int hdrLen  = _hdr->getHdrLength(); 
    int len = _hdr->getLength();

    if(_hdr->getAttrType() != TRIP_COMMUNITY)
    {
        cpLog(LOG_ALERT, "Bad Advertise path");
        throw TRIPBadDataException("Bad Advertise path",
                                   __FILE__, __LINE__, 0);
    }

    u_int16_t tLen = 0;
    while(tLen < len)
    {
        u_int32_t comm;
        memcpy(&comm, buf+hdrLen+tLen, 4);
        comm = ntohl(comm);
        tLen += 4;
        _commList.push_back(comm);
    }
}

void 
TRIPCommunity::copyObj(const TRIPCommunity& other)
{
    TRIPRouteAttr::copyObj(other);
    _commList.clear();
    for(CommunityList::const_iterator itr = other._commList.begin();
               itr != other._commList.end(); itr++)
    {
        _commList.push_back(*itr);
    }
}

void 
TRIPCommunity::addCommunity(u_int32_t commValue)
{
    CommunityList::iterator itr = find(_commList.begin(), _commList.end(), commValue);
    if(itr == _commList.end())
    {
        _commList.push_back(commValue);
    }
    setLength(sizeof(u_int32_t) * _commList.size());
}

==> trip-1.0.0.tar.gz: trip-1.0.0/tripstack/tripcore/TRIPCommunity.cxx <==
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TRIPCommunity_cxx_Version = 
"$Id: TRIPCommunity.cxx,v 1.7 2001/03/05 20:21:19 kevinmc Exp $";

#include <algorithm>

#include "TRIPCommunity.hxx"
#include "TRIPUtil.hxx"
#include "TRIPErrorHandler.hxx"
#include "cpLog.h"

int TRIPCommunity::_communityUnitSize=4;

TRIPCommunity::TRIPCommunity(const TRIPCommunity& other) 
{
     copyObj(other);
}

TRIPCommunity& 
TRIPCommunity::operator=(const TRIPCommunity& other)
{
    if(this != &other)
    {
        copyObj(other);
    }
    return(*this);
}

void 
TRIPCommunity::dump(ostream& os) const
{
    os.width(25);
    os << "**Community**" << endl;
    os.width(25);
    int idx = 0;
    for(CommunityList::const_iterator itr = _commList.begin();
               itr != _commList.end(); itr++)
    {
        os << hex << static_cast<int>(*itr) << ",";
        idx++;
        if(idx > 5)
        {
            //Print no more than 5 communities in a line
            os << endl;
            os.width(25);
            idx = 0;
        }
    }
}

void 
TRIPCommunity::writeData(int id) const throw(VNetworkException&)
{
    TRIPRouteAttr::writeData(id);
    assert(_commList.size());
    u_int32_t* cArray  = new u_int32_t[_commList.size()];
    int i = 0;
    for(CommunityList::const_iterator itr = _commList.begin();
               itr != _commList.end(); itr++)
    {
        cArray[i++] = (*itr);
    }
    TRIPUtil::writeData(id, cArray, sizeof(u_int32_t)*_commList.size());

    delete []cArray;
}

TRIPCommunity::TRIPCommunity(const char* buf) 
        throw (TRIPBadDataException&)
    : TRIPRouteAttr(buf)
{

    if(_hdr->getAttrType() != TRIP_COMMUNITY)
    {
        throw TRIPBadDataException("Bad community attribute",
                                   __FILE__, __LINE__, TRIP_ERR_UPDATE);
    }

    int hdrLen  = _hdr->getHdrLength(); 
    int len = _hdr->getLength();
    u_int16_t tLen = 0;
    //Loop throw all the communities in the attribute and build
    //a community list
    while(tLen < len)
    {
        u_int32_t comm;
        memcpy(&comm, buf+hdrLen+tLen, 4);
        comm = ntohl(comm);
        tLen += 4;
        _commList.push_back(comm);
    }
}

void 
TRIPCommunity::copyObj(const TRIPCommunity& other)
{
    TRIPRouteAttr::copyObj(other);
    _commList.clear();
    for(CommunityList::const_iterator itr = other._commList.begin();
               itr != other._commList.end(); itr++)
    {
        _commList.push_back(*itr);
    }
}

void 
TRIPCommunity::addCommunity(u_int32_t commValue)
{
    CommunityList::iterator itr = find(_commList.begin(), _commList.end(), commValue);
    if(itr == _commList.end())
    {
        _commList.push_back(commValue);
        setLength(_communityUnitSize * _commList.size());
    }
}

bool
TRIPCommunity::containsCommunity(u_int32_t commValue) const
{
    CommunityList::const_iterator itr;
    for(itr = _commList.begin(); itr != _commList.end(); itr++)
    {
        if((*itr) == commValue) return true;
    }
    return false;
}

bool 
TRIPCommunity::isEqualAttribute(const TRIPRouteAttr& attr) const
{
    assert(attr.getType() == TRIP_COMMUNITY);
    const TRIPCommunity* other = dynamic_cast<const TRIPCommunity*>(&attr);
    assert(other);
    if(!other)
    {
        return false;
    }
    return (TRIPRouteAttr::isEqualAttribute(attr) &&
            (_commList == other->_commList)); 
}

void
TRIPCommunity::aggregate(Sptr<TRIPRouteAttr> attr)
    throw (TRIPBadDataException&)
{
    assert(attr->getType() == TRIP_COMMUNITY);
    Sptr<TRIPCommunity> otherComm;
    otherComm.dynamicCast(attr);
    if (otherComm.getPtr() != 0)
    {
        const CommunityList& otherList = otherComm->getCommunityList();
        if (otherList.size() > 0) // no Communities, no need to aggregate
        {
            CommunityList::const_iterator itr;
            for (itr = otherList.begin(); itr != otherList.end(); itr++)
            {
                if (containsCommunity((*itr)) == false)
                {
                    addCommunity((*itr));
                }
            }
        }
    }
    else
    {
        throw TRIPBadDataException("Invalid attribute", __FILE__, 
                                                    __LINE__, TRIP_ERR_UPDATE);
    }
}
